"""Type stubs for the Rust module."""

def run_tests(pytest_args: list[str] | None = ...) -> None:
    """Run tests using the Rust implementation."""
    ...

def main_cli_with_args(argv: list[str]) -> None:
    """Main CLI entry point with arguments."""
    ...
